/**
List of text file extensions.

@example
```
import textExtensions = require('text-extensions');

console.log(textExtensions);
//=> ['asp', 'aspx', …]
```
*/
declare const textExtensions: readonly string[];

export = textExtensions;
