import type { RequestMethod } from "./RequestMethod";
import type { Url } from "./Url";
import type { RequestParameters } from "./RequestParameters";
export type EndpointOptions = RequestParameters & {
    method: RequestMethod;
    url: Url;
};
