'use strict'

function createConventionalChangelogOpts (parserOpts, writerOpts) {
  return {
    parserOpts,
    writerOpts
  }
}

module.exports.createConventionalChangelogOpts = createConventionalChangelogOpts
